import uuid
import logging

import slack_sdk

from typing import Callable, Optional, List
from ppa import ui

from . import exceptions
from .records import Channel, ChannelTypes


logger = logging.getLogger(__name__)

VALID_TYPES = List[Channel]


def _get_cell_value(d, k):
    return str(d.get(k)) if d.get(k) is not None else ""


def _create_client(func: Callable) -> Callable:
    def _execute(token: str, channel: Channel, *args, **kwargs):
        if not isinstance(token, str):
            raise_with_message(exceptions.InvalidToken, "The token must be provided as a string.")
        elif isinstance(channel, list):
            raise_with_message(
                exceptions.BadChannel,
                f"slack.messages.{func.__name__} only accepts a single channel, {len(channel)} were provided",
            )

        client = slack_sdk.WebClient(token)
        try:
            return func(client, channel, *args, **kwargs)

        except slack_sdk.errors.SlackApiError as e:
            error = e.response.get("error")
            if error == "invalid_auth":
                raise_with_message(exceptions.InvalidToken, "The token provided is invalid.")
            elif error == "channel_not_found":
                raise_with_message(exceptions.BadChannel, f"The supplied slack channel: {channel['name']} cannot be found.")
            elif error == "missing_scope":
                raise_with_message(
                    exceptions.MissingScope,
                    f"Your slack app is missing the {e.response.get('needed')} bot token scope, please add this to your slack app.",
                )
            elif error == "not_in_channel":
                raise_with_message(
                    exceptions.AppNotInChannel,
                    f"The slack app associated with the supplied token is not in channel '{channel['name']}'.",
                )
            raise e

    return _execute


@_create_client
def _get_channels(token: str, types: ChannelTypes) -> List[Channel]:
    # note: you can only see private slack channels that your slack app has been added to
    response = token.conversations_list(exclude_archived=True, types=types.value)

    all_channels = []
    for page in response:
        all_channels += [channel for channel in vars(page)["data"]["channels"]]

    return all_channels


def _output_table(text: str, channels: List[Channel], row_generator: Callable, table_header: List[str]) -> None:
    ui.output_table(text, header=table_header, data=[row_generator(channel) for channel in channels])


def _input_table(
    text: str,
    resources: VALID_TYPES,
    row_generator: Callable,
    table_header: List[str],
    minimum: Optional[int] = None,
    maximum: Optional[int] = None,
) -> VALID_TYPES:

    table_element_id, error_element_id = list(map(lambda _: str(uuid.uuid4()), {0, 1}))

    while True:
        indices = ui.input_table(
            text, header=table_header, data=[row_generator(resource) for resource in resources], element_id=table_element_id
        )
        count = len(indices)
        if minimum and count < minimum:
            ui.output_error(f"Please choose at least {minimum} record(s).", element_id=error_element_id)
            continue
        if maximum and count > maximum:
            ui.output_error(f"Please choose no more than {maximum} record(s).", element_id=error_element_id)
            continue
        break

    if count == 1:
        return resources[indices[0]]
    else:
        return [resources[i] for i in indices]


def raise_with_message(exception, message):
    logger.error(message)
    ui.output_error(message)
    raise exception(message)
