"""Channels

This module contains actions for viewing and selecting the channels that are available within your workspace.

!!! tip "Before you Start"

    Ensure you have created a PPA Slack app, & enabled the relevant slack app scopes.

    See the [Integration](../integration/) page for more information

!!! warning "Supplying Sensitive Data"
    You should ___always___ use a PPA Vault integration to provide sensitive data to a plugin action.
"""
from typing import List, Optional, Union

from playbook_tools import tables

from ._client import _get_channels, _input_table, _output_table, _get_cell_value
from .records import Channel, ChannelTypes


TABLE_HEADER = ["Name", "Topic", "Creator", "ID"]


def _generate_row(channel):
    return [
        _get_cell_value(channel, "name"),
        _get_cell_value(channel.get("topic", {}), "value"),
        _get_cell_value(channel, "creator"),
        _get_cell_value(channel, "id"),
    ]


def get_all(token: str) -> List[Channel]:
    """Get all the private and public channels from your slack workspace.

!!! question "Note"
    Your Slack app has to be added to a private channel for that channel to be visible.

!!! question "Minimum Plugin Version: 2.0.1"

??? info "Input"

    - __token:__ The slack app / bot user [token](https://api.slack.com/authentication/token-types#granular_bot)

??? success "Output"

    A list of [Channels](../inputs_outputs/#channel).

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels
    ```
    """
    return _get_channels(token, ChannelTypes.public_private)


def get_public(token: str) -> List[Channel]:
    """Get the public channels from your slack workspace.

!!! question "Minimum Plugin Version: 2.0.1"

??? info "Input"

    - __token:__ The slack app / bot user [token](https://api.slack.com/authentication/token-types#granular_bot)

??? success "Output"

    A list of [Channels](../inputs_outputs/#channel).

??? example

    ```YAML linenums="1"
    - slack.channels.get_public:
      load:
        token: slack_bot_token
      save: public_channels
    ```
    """
    return _get_channels(token, ChannelTypes.public)


def get_private(token: str) -> List[Channel]:
    """Get the private channels from your slack workspace.

!!! question "Minimum Plugin Version: 2.0.1"

!!! question "Note"
    Your Slack app has to be added to a private channel for that channel to be visible.

??? info "Input"

    - __token:__ The slack app / bot user [token](https://api.slack.com/authentication/token-types#granular_bot)

??? success "Output"

    A list of [Channels](../inputs_outputs/#channel).

??? example

    ```YAML linenums="1"
    - slack.channels.get_private:
      load:
        token: slack_bot_token
      save: private_channels
    ```
    """
    return _get_channels(token, ChannelTypes.private)


def display(text: str, channels: Union[Channel, List[Channel]]) -> None:
    """Display channels in a table.

The table will have the following columns:

- Name
- Topic
- Creator
- ID

!!! question "Minimum Plugin Version: 3.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __channels:__ any number of [Channels](../inputs_outputs/#channel).

??? success "Output"
    Nothing is outputted by this action.

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels

    - slack.channels.display:
        text: All channels
      load:
        channels: all_channels
    ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=_generate_row, records=channels)


def output_table(text: str, channels: List[Channel]) -> None:
    """Display channels in a table.

The table will have the following columns:

- Name
- Topic
- Creator
- ID

!!! warning "Deprecation Warning"
    This action will soon be deprecated in favour of [display](#display).

??? info "Input"

    - __text:__ the title of the table

    - __channels:__ any number of [Channels](../inputs_outputs/#channel).

??? success "Output"
    Nothing is outputted by this action.

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels

    - slack.channels.output_table:
        text: All channels
      load:
        channels: all_channels
    ```
    """
    _output_table(text, channels, _generate_row, TABLE_HEADER)


def input_table(
    text: str, channels: List[Channel], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> List[Channel]:
    """Display channels in a table, & allow the task operator to make a selection.

The table will have the following columns:

- Name
- Topic
- Creator
- ID


!!! warning "Deprecation Warning"
    This action will soon be deprecated in favour of [select](#select).

??? info "Input"

    - __text:__ the title of the table

    - __channels:__ any number of [Channels](../inputs_outputs/#channel).

    - __minimum:__ the minimum number of acceptable selections (Optional)

    - __maximum:__ the maximum number of acceptable selections (Optional)

??? success "Output"
    A list of the selected [Channels](../inputs_outputs/#channel).

??? tip
    - If neither a _minimum_ or _maximum_ is provided, the task operator will be able to submit 0 selections.
    - If _minimum_ or _maximum_ are provided, the operation will repeat until the task operator makes a valid number
      of selections.

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels

    - slack.channels.input_table:
        text: "Select a channel for approval request"
        maximum: 1
        minimum: 1
      load:
        channels: all_channels
      save:
        approval_channel
    ```
    """
    return _input_table(text, channels, _generate_row, TABLE_HEADER, minimum, maximum)


def select(
    text: str, channels: List[Channel], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> tables.TableSelection:
    """Display channels in a table, & allow the task operator to make a selection.

The table will have the following columns:

- Name
- Topic
- Creator
- ID

!!! question "Minimum Plugin Version: 3.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __channels:__ any number of [Channels](../inputs_outputs/#channel)

    - __minimum:__ the minimum number of acceptable selections (Optional)

    - __maximum:__ the maximum number of acceptable selections (Optional)

??? success "Output"

    A [Selection](../../../tables/#selection) containing:

    - **total**: the number of selected [Channels](../inputs_outputs/#channel)

    - **all**: a list of selected [Channels](../inputs_outputs/#channel)

    - **first**: the first selected [Channel](../inputs_outputs/#channel)

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels

    - slack.channels.select:
        text: Select Channels
      load:
        channels: all_channels
      save: channel_selection
    ```
    """
    return tables.input_table(
        text, header=TABLE_HEADER, extract=_generate_row, minimum=minimum, maximum=maximum, records=channels
    )


def select_one(text: str, channels: List[Channel]) -> Channel:
    """Display channels in a table, & allow the task operator to make a selection.

The table will have the following columns:

- Name
- Topic
- Creator
- ID

!!! question "Minimum Plugin Version: 3.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __channels:__ any number of [Channels](../inputs_outputs/#channel).

??? success "Output"
    A single [Channel](../inputs_outputs/#channel).

??? example

    ```YAML linenums="1"
    - slack.channels.get_all:
      load:
        token: slack_bot_token
      save: all_channels

    - slack.channels.select_one:
        text: Select a Channel
      load:
        channels: all_channels
      save: one_channel
    ```
    """
    return select(text, channels, minimum=1, maximum=1)["first"]
